@@ -209,20 +209,21 @@ void CCharTip::showTip()
     details+="</table>";
     itsLabel->setText(details);
 
-    QPixmap pix((int)(itsItem.width()*2.5), (int)(itsItem.height()*2.5));
     QList<CFcEngine::TRange> range;
     range.append(CFcEngine::TRange(itsItem.ucs4, 0));
 
-    QColor prevBgndCol(CFcEngine::bgndCol());
+    QColor bgnd(Qt::white);
+    bgnd.setAlpha(0);
 
-    CFcEngine::setBgndCol(palette().color(QPalette::Active, QPalette::Background));
-    if(CFcEngine::instance()->draw(itsParent->itsCurrentUrl, pix.width(), pix.height(), pix,
-                                   itsParent->itsCurrentFace-1, false, range,
-                                   NULL, itsParent->itsFontName, itsParent->itsStyleInfo))
-        itsPixmapLabel->setPixmap(pix);
+    QImage img=itsParent->engine()->draw(itsParent->itsFontName, itsParent->itsStyleInfo,
+                                         itsParent->itsCurrentFace-1, palette().text().color(), bgnd,
+                                         (int)(itsItem.width()*2.5), (int)(itsItem.height()*2.5), false, range, NULL);
+
+    if(!img.isNull())
+        itsPixmapLabel->setPixmap(QPixmap::fromImage(img));
     else
         itsPixmapLabel->setPixmap(QPixmap());
-    CFcEngine::setBgndCol(prevBgndCol);
+
     itsTimer->disconnect(this);
     connect(itsTimer, SIGNAL(timeout()), this, SLOT(hideTip()));
     itsTimer->setSingleShot(true);
@@ -249,11 +250,13 @@ void CCharTip::reposition()
     QPoint pos(rect.center());
     QRect  desk(KGlobalSettings::desktopGeometry(rect.center()));
 
-    if (rect.center().x() + width() > desk.right())
+    if ((rect.center().x() + width()) > desk.right())
+    {
         if (pos.x() - width() < 0)
             pos.setX(0);
         else
             pos.setX( pos.x() - width() );
+    }
     // should the tooltip be shown above or below the ivi ?
     if (rect.bottom() + height() > desk.bottom())
         pos.setY( rect.top() - height() );
