@@ -22,14 +22,12 @@
 #define _KCM_XINERAMA_H
 
 #include <kcmodule.h>
-#include <Qt3Support/Q3PtrList>
 #include <QTimer>
 
 #include "xineramawidget.h"
 
 class KConfig;
 class QWidget;
-class QStringList;
 
 class KCMXinerama : public KCModule {
 	Q_OBJECT
