@@ -102,7 +102,6 @@ extern char **environ;
 # define Jmp_buf sigjmp_buf
 #endif
 
-#include <utmp.h>
 #ifdef HAVE_UTMPX
 # include <utmpx.h>
 # define STRUCTUTMP struct utmpx
@@ -113,15 +112,16 @@ extern char **environ;
 # define ENDUTENT endutxent
 # define ut_time ut_tv.tv_sec
 #else
+# include <utmp.h>
 # define STRUCTUTMP struct utmp
 # define UTMPNAME utmpname
 # define SETUTENT setutent
 # define GETUTENT getutent
 # define PUTUTLINE pututline
 # define ENDUTENT endutent
-#endif
-#ifndef HAVE_STRUCT_UTMP_UT_USER
-# define ut_user ut_name
+# ifndef HAVE_STRUCT_UTMP_UT_USER
+#  define ut_user ut_name
+# endif
 #endif
 #ifndef WTMP_FILE
 # ifdef _PATH_WTMPX
@@ -242,7 +242,7 @@ struct display {
 
 	/* basic display information */
 	char *name;                 /* DISPLAY name -- also referenced in hstent */
-	char *class2;               /* display class (may be NULL) */
+	char *class2;               /* display class (may be 0) */
 	int displayType;            /* location/origin/lifetime */
 	CfgArr cfg;                 /* config data array */
 
@@ -270,8 +270,8 @@ struct display {
 	char *remoteHost;           /* for X -query type remote login */
 	/* XDMCP state */
 	unsigned sessionID;         /* ID of active session */
-	ARRAY8 peer;                /* display peer address */
-	ARRAY8 from;                /* XDMCP port of display */
+	ARRAY8 peer;                /* display peer address (sockaddr) */
+	ARRAY8 from;                /* XDMCP peer address (sockaddr) */
 	unsigned displayNumber;     /* numerical part of name */
 	int useChooser;             /* Run the chooser for this display */
 	ARRAY8 clientAddr;          /* for chooser picking */
@@ -284,10 +284,7 @@ struct display {
 
 	int idleTimeout;            /* abort login after that time */
 
-	unsigned short *authNameLens;  /* authorization protocol name lens */
-
 	/* information potentially derived from resources */
-	int authNameNum;            /* number of protocol names */
 	Xauth **authorizations;     /* authorization data */
 	int authNum;                /* number of authorizations */
 	char *authFile;             /* file to store authorization in */
@@ -303,10 +300,11 @@ struct display {
 #define dTransient      0	/* display removed when session exits */
 
 #ifdef XDMCP
-#define d_origin     8
+#define d_origin     24
 #else
-#define d_origin     0 /* clever, huh? :) */
+#define d_origin     16 /* clever, huh? :) */
 #endif
+#define dFromCommand   16	/* started via command socket */
 #define dFromXDMCP      8	/* started with XDMCP */
 #define dFromFile       0	/* started via entry in servers file */
 
@@ -375,15 +373,15 @@ extern char *progname;
 extern char *prog;
 extern time_t now;
 extern SdRec sdRec;
-void startDisplay( struct display *d );
 void startDisplayP2( struct display *d );
 void stopDisplay( struct display *d );
 #if !defined(HAVE_SETPROCTITLE) && !defined(NOXDMTITLE)
 void setproctitle( const char *fmt, ... );
 #endif
+void wakeDisplays( void );
 void switchToX( struct display *d );
 void setNLogin( struct display *d,
-                const char *nuser, const char *npass, char *nargs,
+                const char *nuser, const char *npass, const char *nargs,
                 int rl );
 void cancelShutdown( void );
 int TTYtoVT( const char *tty );
@@ -417,6 +415,7 @@ int anyRunningDisplays( void );
 int anyReserveDisplays( void );
 int idleReserveDisplays( void );
 int startReserveDisplay( int lt );
+const char *displayName( struct display * );
 
 /* in reset.c */
 void pseudoReset( void );
@@ -440,7 +439,7 @@ char **baseEnv( char **env, const char *user );
 char **inheritEnv( char **env, const char **what );
 char **systemEnv( char **env, const char *user );
 int source( char **env, const char *file, const char *arg );
-void manageSession( struct display *d );
+void manageSession( void );
 
 extern GTalk mstrtalk, grttalk;
 extern GProc grtproc;
@@ -561,6 +560,8 @@ int reader( int fd, void *buf, int len );
 int writer( int fd, const void *buf, int len );
 int fGets( char *buf, int max, FILE *f );
 time_t mTime( const char *fn );
+void randomStr( char *s );
+int hexToBinary( char *out, const char *in );
 void listSessions( int flags, struct display *d, void *ctx,
                    void (*emitXSess)( struct display *, struct display *, void * ),
                    void (*emitTTYSess)( STRUCTUTMP *, struct display *, void * ) );
