@@ -319,6 +319,60 @@ displace( const char *fn )
 }
 
 
+static char *curName;
+
+/* Create a new file in KDMCONF */
+static FILE *
+createFile( const char *fn, int mode )
+{
+	FILE *f;
+
+	free( curName );
+	ASPrintf( &curName, "%s/%s", newdir, fn );
+	displace( curName );
+	if (!(f = fopen( curName, "w" ))) {
+		fprintf( stderr, "Cannot create %s\n", curName );
+		exit( 1 );
+	}
+	chmod( curName, mode );
+	return f;
+}
+
+static void
+writeError()
+{
+	fprintf( stderr, "Warning: cannot write %s (disk full?)\n", curName );
+	unlink( curName );
+	exit( 1 );
+}
+
+static void
+fputs_( const char *str, FILE *f )
+{
+	if (fputs( str, f ) == EOF)
+		writeError();
+}
+
+static void
+ATTR_PRINTFLIKE(2, 3)
+fprintf_( FILE *f, const char *fmt, ... )
+{
+	va_list args;
+
+	va_start( args, fmt );
+	if (vfprintf( f, fmt, args ) < 0)
+		writeError();
+	va_end( args );
+}
+
+static void
+fclose_( FILE *f )
+{
+	if (fclose( f ) == EOF)
+		writeError();
+}
+
+
 static char *
 locate( const char *exe )
 {
@@ -464,8 +518,8 @@ writeKdmrc( FILE *f )
 
 	putFqVal( "General", "ConfigVersion", RCVERSTR );
 	for (cs = config; cs; cs = cs->next) {
-		fprintf( f, "%s[%s]\n",
-		         cs->comment ? cs->comment : "\n", cs->name );
+		fprintf_( f, "%s[%s]\n",
+		          cs->comment ? cs->comment : "\n", cs->name );
 		for (ce = cs->ents; ce; ce = ce->next) {
 			if (ce->spec->comment) {
 				cmt = ce->spec->comment;
@@ -483,8 +537,8 @@ writeKdmrc( FILE *f )
 			} else
 				cmt = "";
 		  havit:
-			fprintf( f, "%s%s%s=%s\n",
-			         cmt, ce->active ? "" : "#", ce->spec->key, ce->value );
+			fprintf_( f, "%s%s%s=%s\n",
+			          cmt, ce->active ? "" : "#", ce->spec->key, ce->value );
 		}
 	}
 }
@@ -595,26 +649,9 @@ static const char def_startup[] =
 "#\n"
 "#chown $USER /dev/console\n"
 "\n"
-#ifdef _AIX
-"# We create a pseudodevice for finger.  (host:0 becomes xdm/host_0)\n"
-"# Without it, finger errors out with \"Cannot stat /dev/host:0\".\n"
-"#\n"
-"#devname=`echo $DISPLAY | cut -c1-8`\n"
-"#if [ ! -d /dev/xdm ]; then\n"
-"#  mkdir /dev/xdm\n"
-"#  chmod 755 /dev/xdm\n"
-"#fi\n"
-"#touch /dev/xdm/$devname\n"
-"#chmod 644 /dev/xdm/$devname\n"
-"#exec sessreg -a -l xdm/$devname -h \"`echo $DISPLAY | cut -d: -f1`\""
-#else
-"#exec sessreg -a -l $DISPLAY -h \"`echo $DISPLAY | cut -d: -f1`\""
-# ifdef BSD
-" -x " KDMCONF "/Xservers"
-# endif
-#endif /* _AIX */
-" $USER\n"
-"\n# NOTE: The session is aborted if the last command returns non-zero.\n";
+"# XDM configurations typically have sessreg here. KDM has it built-in.\n"
+"\n"
+"# NOTE: The session is aborted if the last command returns non-zero.\n";
 
 static const char def_reset[] =
 "#! /bin/sh\n"
@@ -626,16 +663,7 @@ static const char def_reset[] =
 "#chown root /dev/console\n"
 "#chmod 622 /dev/console\n"
 "\n"
-#ifdef _AIX
-"#devname=`echo $DISPLAY | cut -c1-8`\n"
-"#exec sessreg -d -l xdm/$devname -h \"`echo $DISPLAY | cut -d: -f1`\""
-#else
-"#exec sessreg -d -l $DISPLAY -h \"`echo $DISPLAY | cut -d: -f1`\""
-# ifdef BSD
-" -x " KDMCONF "/Xservers"
-# endif
-#endif /* _AIX */
-" $USER\n";
+"# XDM configurations typically have sessreg here. KDM has it built-in.\n";
 
 static const char def_session1[] =
 "#! /bin/sh\n"
@@ -726,24 +754,6 @@ static const char def_background[] =
 "WallpaperList=\n"
 "WallpaperMode=Scaled\n";
 
-/* Create a new file in KDMCONF */
-static FILE *
-createFile( const char *fn, int mode )
-{
-	char *nname;
-	FILE *f;
-
-	ASPrintf( &nname, "%s/%s", newdir, fn );
-	displace( nname );
-	if (!(f = fopen( nname, "w" ))) {
-		fprintf( stderr, "Cannot create %s\n", nname );
-		exit( 1 );
-	}
-	chmod( nname, mode );
-	free( nname );
-	return f;
-}
-
 /* Create a copy of a file under KDMCONF and fill it */
 static void
 writeCopy( const char *fn, int mode, time_t stamp, const char *buf, size_t len )
@@ -759,8 +769,11 @@ writeCopy( const char *fn, int mode, time_t stamp, const char *buf, size_t len )
 		fprintf( stderr, "Cannot create %s\n", nname );
 		exit( 1 );
 	}
-	write( fd, buf, len );
-	close( fd );
+	if (write( fd, buf, len ) != (ssize_t)len || close( fd ) < 0) {
+		fprintf( stderr, "Cannot write %s (disk full?)\n", nname );
+		unlink( nname );
+		exit( 1 );
+	}
 	if (stamp) {
 		utim.actime = utim.modtime = stamp;
 		utime( nname, &utim );
@@ -1022,14 +1035,14 @@ static void
 writeFile( const char *tname, int mode, const char *cont )
 {
 	FILE *f = createFile( tname + sizeof(KDMCONF), mode );
-	fputs( cont, f );
-	fclose( f );
+	fputs_( cont, f );
+	fclose_( f );
 	addedFile( tname );
 }
 
 
 static void
-handleBgCfg( Entry *ce, Section *cs ATTR_UNUSED )
+handleBgCfg( Entry *ce, Section *cs )
 {
 	if (!ce->active) /* can be only the X-*-Greeter one */
 		writeFile( def_BackgroundCfg, 0644, def_background );
@@ -1555,6 +1568,8 @@ upd_servercmd( Entry *ce, Section *cs ATTR_UNUSED )
 		addStr( &sa, "-br" );
 	if (strstr( buf, "\n-novtswitch " ))
 		addStr( &sa, "-novtswitch" );
+	if (strstr( buf, "\n-quiet " ))
+		addStr( &sa, "-quiet" );
 	ce->value = joinArgs( sa );
 	ce->written = True;
 }
@@ -1942,7 +1957,7 @@ upd_maxshowuid( Entry *ce, Section *cs ATTR_UNUSED )
 }
 
 static void
-upd_hiddenusers( Entry *ce, Section *cs ATTR_UNUSED )
+upd_hiddenusers( Entry *ce, Section *cs )
 {
 	char *nv;
 	const char *msu, *pt, *et;
@@ -2044,8 +2059,15 @@ copyPlainFile( const char *from, const char *to )
 
 	if (readFile( &file, from )) {
 		if ((fd = open( to, O_WRONLY | O_CREAT | O_EXCL, 0644 )) >= 0) {
-			write( fd, file.buf, file.eof - file.buf );
-			close( fd );
+			size_t len = file.eof - file.buf;
+			if (write( fd, file.buf, len ) != (ssize_t)len) {
+				fprintf( stderr, "Warning: cannot write %s (disk full?)\n", to );
+				unlink( to );
+			}
+			if (close( fd ) < 0) {
+				fprintf( stderr, "Warning: cannot write %s (disk full?)\n", to );
+				unlink( to );
+			}
 		} else if (errno != EEXIST)
 			fprintf( stderr, "Warning: cannot create %s\n", to );
 		freeBuf( &file );
@@ -2809,8 +2831,10 @@ fprintfLineWrap( FILE *f, const char *msg, ... )
 		col++;
 	}
 	free( txt );
-	fputs( ftxt, f );
-	free( ftxt );
+	if (ftxt) {
+		fputs_( ftxt, f );
+		free( ftxt );
+	}
 }
 
 
@@ -3100,62 +3124,62 @@ int main( int argc, char **argv )
 	defminuid = "1000";
 	defmaxuid = "65000";
 #endif
-	for (i = 0; i < CONF_MAX_PRIO; i++)
+	for (i = 0; i <= CONF_MAX_PRIO; i++)
 		for (cs = config; cs; cs = cs->next)
 			for (ce = cs->ents; ce; ce = ce->next)
 				if (ce->spec->func && i == ce->spec->prio)
 					ce->spec->func( ce, cs );
 	f = createFile( "kdmrc", kdmrcmode );
 	writeKdmrc( f );
-	fclose( f );
+	fclose_( f );
 
 	f = createFile( "README", 0644 );
-	fprintf( f,
+	fprintf_( f,
 "This automatically generated configuration consists of the following files:\n" );
-	fprintf( f, "- " KDMCONF "/kdmrc\n" );
+	fprintf_( f, "- " KDMCONF "/kdmrc\n" );
 	for (fp = aflist; fp; fp = fp->next)
-		fprintf( f, "- %s\n", fp->str );
+		fprintf_( f, "- %s\n", fp->str );
 	if (use_destdir && !no_in_notice)
 		fprintfLineWrap( f,
 "All files destined for " KDMCONF " were actually saved in %s; "
 "this config will not be workable until moved in place.\n", newdir );
 	if (uflist || eflist || cflist || lflist) {
-		fprintf( f,
+		fprintf_( f,
 "\n"
 "This config was derived from existing files. As the used algorithms are\n"
 "pretty dumb, it may be broken.\n" );
 		if (uflist) {
-			fprintf( f,
+			fprintf_( f,
 "Information from these files was extracted:\n" );
 			for (fp = uflist; fp; fp = fp->next)
-				fprintf( f, "- %s\n", fp->str );
+				fprintf_( f, "- %s\n", fp->str );
 		}
 		if (lflist) {
-			fprintf( f,
+			fprintf_( f,
 "These files were directly incorporated:\n" );
 			for (fp = lflist; fp; fp = fp->next)
-				fprintf( f, "- %s\n", fp->str );
+				fprintf_( f, "- %s\n", fp->str );
 		}
 		if (cflist) {
-			fprintf( f,
+			fprintf_( f,
 "These files were copied verbatim:\n" );
 			for (fp = cflist; fp; fp = fp->next)
-				fprintf( f, "- %s\n", fp->str );
+				fprintf_( f, "- %s\n", fp->str );
 		}
 		if (eflist) {
-			fprintf( f,
+			fprintf_( f,
 "These files were copied with modifications:\n" );
 			for (fp = eflist; fp; fp = fp->next)
-				fprintf( f, "- %s\n", fp->str );
+				fprintf_( f, "- %s\n", fp->str );
 		}
 		if (!no_backup && !use_destdir)
-			fprintf( f,
+			fprintf_( f,
 "Old files that would have been overwritten were renamed to <oldname>.bak.\n" );
 	}
-	fprintf( f,
+	fprintf_( f,
 "\nTry 'genkdmconf --help' if you want to generate another configuration.\n"
 "\nYou may delete this README.\n" );
-	fclose( f );
+	fclose_( f );
 
 	return 0;
 }
