@@ -45,6 +45,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <QLabel>
 #include <QLineEdit>
 #include <QMenu>
+#include <QStylePainter>
 #include <QStyle>
 #include <QTreeWidget>
 #include <QTreeWidgetItem>
@@ -98,7 +99,6 @@ KDMShutdownBase::complete( QWidget *prevWidget )
 		                          KGreeterPlugin::Shutdown );
 		verify->selectPlugin( curPlugin );
 		box->addLayout( verify->getLayout() );
-		verify->getLayout()->setParent( box );
 		QAction *action = new QAction( this );
 		action->setShortcut( Qt::ALT+Qt::Key_A );
 		connect( action, SIGNAL(triggered( bool )), SLOT(slotActivatePlugMenu()) );
@@ -108,7 +108,6 @@ KDMShutdownBase::complete( QWidget *prevWidget )
 
 	QBoxLayout *hlay = new QHBoxLayout();
 	box->addLayout( hlay );
-	hlay->setParent( box );
 	hlay->addStretch( 1 );
 	if (mayOk) {
 		okButton = new KPushButton( KStandardGuiItem::ok(), this );
@@ -120,7 +119,7 @@ KDMShutdownBase::complete( QWidget *prevWidget )
 	}
 	if (maySched) {
 		KPushButton *schedButton =
-			new KPushButton( KGuiItem( i18n("&Schedule...") ), this );
+			new KPushButton( KGuiItem( i18nc("@action:inmenu verb", "&Schedule...") ), this );
 		schedButton->setSizePolicy( fp );
 		hlay->addWidget( schedButton );
 		hlay->addStretch( 1 );
@@ -224,7 +223,7 @@ KDMShutdownBase::verifySetUser( const QString & )
 
 
 static void
-doShutdown( int type, const char *os )
+doShutdown( int type, const QString &os )
 {
 	gSet( 1 );
 	gSendInt( G_Shutdown );
@@ -233,11 +232,26 @@ doShutdown( int type, const char *os )
 	gSendInt( 0 );
 	gSendInt( SHUT_FORCE );
 	gSendInt( 0 ); /* irrelevant, will timeout immediately anyway */
-	gSendStr( os );
+	gSendStr( os.toUtf8().data() );
 	gSet( 0 );
 }
 
 
+static bool
+getBootOptions( QStringList *options, int *defaultTarget, int *oldTarget )
+{
+	bool ret = false;
+	gSet( 1 );
+	gSendInt( G_ListBootOpts );
+	if (gRecvInt() == BO_OK) {
+		*options = qStringList( gRecvStrArr( 0 ) );
+		*defaultTarget = gRecvInt();
+		*oldTarget = gRecvInt();
+		ret = true;
+	}
+	gSet( 0 );
+	return ret;
+}
 
 KDMShutdown::KDMShutdown( int _uid, QWidget *_parent )
 	: inherited( _uid, _parent )
@@ -246,7 +260,6 @@ KDMShutdown::KDMShutdown( int _uid, QWidget *_parent )
 
 	QHBoxLayout *hlay = new QHBoxLayout();
 	box->addLayout( hlay );
-	hlay->setParent( box );
 
 	howGroup = new QGroupBox( i18n("Shutdown Type"), this );
 	hlay->addWidget( howGroup, 0, Qt::AlignTop );
@@ -265,31 +278,24 @@ KDMShutdown::KDMShutdown( int _uid, QWidget *_parent )
 	connect( rb, SIGNAL(doubleClicked()), SLOT(accept()) );
 	connect( restart_rb, SIGNAL(doubleClicked()), SLOT(accept()) );
 
-	gSet( 1 );
-	gSendInt( G_ListBootOpts );
-	if (gRecvInt() == BO_OK) { /* XXX show dialog on failure */
-		char **tlist = gRecvStrArr( 0 );
-		int defaultTarget = gRecvInt();
-		oldTarget = gRecvInt();
+	QStringList options;
+	int defaultTarget;
+	if (getBootOptions( &options, &defaultTarget, &oldTarget )) { /* XXX show dialog on failure */
 		targets = new QComboBox();
-		for (int i = 0; tlist[i]; i++)
-			targets->addItem( QString::fromLocal8Bit( tlist[i] ) );
-		freeStrArr( tlist );
+		targets->addItems( options );
 		targets->setCurrentIndex( oldTarget == -1 ? defaultTarget : oldTarget );
 		QHBoxLayout *hb = new QHBoxLayout();
 		hwlay->addLayout( hb );
-		hb->setParent( hwlay );
 		hb->addSpacing(
 			style()->pixelMetric( QStyle::PM_ExclusiveIndicatorWidth )
 			+ hb->spacing() );
 		hb->addWidget( targets );
 		connect( targets, SIGNAL(activated( int )), SLOT(slotTargetChanged()) );
 	}
-	gSet( 0 );
 
 	howGroup->setSizePolicy( fp );
 
-	schedGroup = new QGroupBox( i18n("Scheduling"), this );
+	schedGroup = new QGroupBox( i18nc("@title:group ... of shutdown", "Scheduling"), this );
 	hlay->addWidget( schedGroup, 0, Qt::AlignTop );
 
 	le_start = new QLineEdit( schedGroup );
@@ -416,8 +422,7 @@ KDMShutdown::scheduleShutdown( QWidget *_parent )
 		uid = ret == Authed ? 0 : -1;
 	} else
 		uid = -1;
-	if (os)
-		free( os );
+	free( os );
 	KDMShutdown( uid, _parent ).exec();
 }
 
@@ -455,21 +460,18 @@ void KDMDelayedPushButton::setDelayedMenu( QMenu *p )
 
 KDMSlimShutdown::KDMSlimShutdown( QWidget *_parent )
 	: inherited( _parent )
-	, targetList( 0 )
 {
 	QHBoxLayout *hbox = new QHBoxLayout( this );
 
 	QFrame *lfrm = new QFrame( this );
-	lfrm->setFrameStyle( QFrame::Panel | QFrame::Sunken );
 	hbox->addWidget( lfrm, Qt::AlignCenter );
 	QLabel *icon = new QLabel( lfrm );
-	icon->setPixmap( QPixmap( KStandardDirs::locate( "data", "kdm/pics/shutdown.jpg" ) ) );
+	icon->setPixmap( QPixmap( KStandardDirs::locate( "data", "kdm/pics/shutdown.png" ) ) );
 	icon->setFixedSize( icon->sizeHint() );
 	lfrm->setFixedSize( icon->sizeHint() );
 
 	QVBoxLayout *buttonlay = new QVBoxLayout();
 	hbox->addLayout( buttonlay );
-	buttonlay->setParent( hbox );
 
 	buttonlay->addStretch( 1 );
 
@@ -481,35 +483,28 @@ KDMSlimShutdown::KDMSlimShutdown( QWidget *_parent )
 	buttonlay->addSpacing( KDialog::spacingHint() );
 
 	KDMDelayedPushButton *btnReboot = new
-	KDMDelayedPushButton( KGuiItem( i18n("&Restart Computer"), "system-restart" ), this );
+	KDMDelayedPushButton( KGuiItem( i18n("&Restart Computer"), "system-reboot" ), this );
 	buttonlay->addWidget( btnReboot );
 	connect( btnReboot, SIGNAL(clicked()), SLOT(slotReboot()) );
 
-	gSet( 1 );
-	gSendInt( G_ListBootOpts );
-	if (gRecvInt() == BO_OK) {
-		targetList = gRecvStrArr( 0 );
-		/*int def =*/ gRecvInt();
-		int cur = gRecvInt();
+	int dummy, cur;
+	if (getBootOptions( &targetList, &dummy, &cur )) {
 		QMenu *targets = new QMenu( this );
-		for (int i = 0; targetList[i]; i++) {
-			QString t( QString::fromLocal8Bit( targetList[i] ) );
+		for (int i = 0; i < targetList.size(); i++)
 			(targets->addAction( i == cur ?
 			                     i18nc( "current option in boot loader",
-			                            "%1 (current)", t ) :
-			                     t ))->setData( i );
-		}
+			                            "%1 (current)", targetList[i] ) :
+			                     targetList[i] ))->setData( i );
 		btnReboot->setDelayedMenu( targets );
 		connect( targets, SIGNAL(triggered( QAction * )),
 		         SLOT(slotReboot( QAction * )) );
 	}
-	gSet( 0 );
 
 	buttonlay->addStretch( 1 );
 
 	if (_scheduledSd != SHUT_NEVER) {
 		KPushButton *btnSched = new
-		KPushButton( KGuiItem( i18n("&Schedule...") ), this );
+		KPushButton( KGuiItem( i18nc("@action:button verb", "&Schedule...") ), this );
 		buttonlay->addWidget( btnSched );
 		connect( btnSched, SIGNAL(clicked()), SLOT(slotSched()) );
 
@@ -527,11 +522,6 @@ KDMSlimShutdown::KDMSlimShutdown( QWidget *_parent )
 	buttonlay->addSpacing( KDialog::spacingHint() );
 }
 
-KDMSlimShutdown::~KDMSlimShutdown()
-{
-	freeStrArr( targetList );
-}
-
 void
 KDMSlimShutdown::slotSched()
 {
@@ -562,14 +552,13 @@ KDMSlimShutdown::slotReboot( QAction *action )
 }
 
 bool
-KDMSlimShutdown::checkShutdown( int type, const char *os )
+KDMSlimShutdown::checkShutdown( int type, const QString &os )
 {
 	reject();
-	dpySpec *sess = fetchSessions( lstRemote | lstTTY );
-	if (!sess && _allowShutdown != SHUT_ROOT)
+	QList<DpySpec> sess = fetchSessions( lstRemote | lstTTY );
+	if (sess.isEmpty() && _allowShutdown != SHUT_ROOT)
 		return true;
 	int ret = KDMConfShutdown( -1, sess, type, os ).exec();
-	disposeSessions( sess );
 	if (ret == Schedule) {
 		KDMShutdown::scheduleShutdown();
 		return false;
@@ -578,20 +567,24 @@ KDMSlimShutdown::checkShutdown( int type, const char *os )
 }
 
 void
-KDMSlimShutdown::externShutdown( int type, const char *os, int uid )
+KDMSlimShutdown::externShutdown( int type, const QString &os, int uid, bool ask )
 {
-	dpySpec *sess = fetchSessions( lstRemote | lstTTY );
-	int ret = KDMConfShutdown( uid, sess, type, os ).exec();
-	disposeSessions( sess );
-	if (ret == Schedule)
-		KDMShutdown( uid ).exec();
-	else if (ret)
-		doShutdown( type, os );
+	QList<DpySpec> sess = fetchSessions( lstRemote | lstTTY );
+	if (ask || !sess.isEmpty() || (uid && _allowShutdown == SHUT_ROOT)) {
+		int ret = KDMConfShutdown( uid, sess, type, os ).exec();
+		if (ret == Schedule) {
+			KDMShutdown( uid ).exec();
+			return;
+		} else if (!ret) {
+			return;
+		}
+	}
+	doShutdown( type, os );
 }
 
 
-KDMConfShutdown::KDMConfShutdown( int _uid, dpySpec *sess, int type, const char *os,
-                                  QWidget *_parent )
+KDMConfShutdown::KDMConfShutdown( int _uid, const QList<DpySpec> &sessions, int type,
+                                  const QString &os, QWidget *_parent )
 	: inherited( _uid, _parent )
 {
 #ifdef HAVE_VTS
@@ -608,13 +601,12 @@ KDMConfShutdown::KDMConfShutdown( int _uid, dpySpec *sess, int type, const char
 	                                  i18n("Switch to Console") :
 #endif
 	                                  i18n("Restart Computer") )
-	                            .arg( os ?
-	                                  i18n("<br/>(Next boot: %1)",
-	                                    QString::fromLocal8Bit( os ) ) :
+	                            .arg( !os.isEmpty() ?
+	                                  i18n("<br/>(Next boot: %1)", os ) :
 	                                  QString() ),
 	                            this ) );
 
-	if (sess) {
+	if (!sessions.isEmpty()) {
 		if (willShut && _scheduledSd != SHUT_NEVER)
 			maySched = true;
 		mayNuke = doesNuke = true;
@@ -632,14 +624,16 @@ KDMConfShutdown::KDMConfShutdown( int _uid, dpySpec *sess, int type, const char
 		lv->setUniformRowHeights( true );
 		lv->setEditTriggers( QAbstractItemView::NoEditTriggers );
 		lv->setColumnCount( 2 );
-		lv->setHeaderLabels( QStringList() << i18n("Session") << i18n("Location") );
+		lv->setHeaderLabels( QStringList()
+			<< i18nc("@title:column", "Session")
+			<< i18nc("@title:column ... of session", "Location") );
 		int ns = 0;
 		QString user, loc;
-		do {
+		foreach (const DpySpec &sess, sessions) {
 			decodeSession( sess, user, loc );
 			new QTreeWidgetItem( lv, QStringList() << user << loc );
-			sess = sess->next, ns++;
-		} while (sess);
+			ns++;
+		}
 		int fw = lv->frameWidth() * 2;
 		int hh = lv->header()->sizeHint().height();
 		int ih = lv->itemDelegate()->sizeHint(
@@ -662,7 +656,7 @@ KDMConfShutdown::KDMConfShutdown( int _uid, dpySpec *sess, int type, const char
 
 
 KDMCancelShutdown::KDMCancelShutdown( int how, int start, int timeout,
-                                      int force, int uid, const char *os,
+                                      int force, int uid, const QString &os,
                                       QWidget *_parent )
 	: inherited( -1, _parent )
 {
@@ -673,20 +667,20 @@ KDMCancelShutdown::KDMCancelShutdown( int how, int start, int timeout,
 			mayNuke = doesNuke = true;
 	}
 	QLabel *lab = new QLabel( mayOk ?
-	                          i18n("Abort pending shutdown:") :
-	                          i18n("No permission to abort pending shutdown:"),
+	                          i18n("Cancel pending shutdown:") :
+	                          i18n("No permission to cancel pending shutdown:"),
 	                          this );
 	box->addWidget( lab );
 	QDateTime qdt;
 	QString strt, end;
 	if (start < time( 0 ))
-		strt = i18n("now");
+		strt = i18nc("start of shutdown:", "now");
 	else {
 		qdt.setTime_t( start );
 		strt = qdt.toString( Qt::LocalDate );
 	}
 	if (timeout == TO_INF)
-		end = i18n("infinite");
+		end = i18nc("timeout of shutdown:", "infinite");
 	else {
 		qdt.setTime_t( timeout );
 		end = qdt.toString( Qt::LocalDate );
@@ -697,24 +691,24 @@ KDMCancelShutdown::KDMCancelShutdown( int how, int start, int timeout,
 		     "\nStart: %3"
 		     "\nTimeout: %4",
 		  uid == -2 ?
-		      i18n("console user") :
+		      i18nc("owner of shutdown:", "console user") :
 		      uid == -1 ?
-		      i18n("control socket") :
+		      i18nc("owner of shutdown:", "control socket") :
 		      KUser( uid ).loginName() ,
 		  how == SHUT_HALT ?
 		      i18n("turn off computer") :
 		      i18n("restart computer") ,
 		  strt, end ,
-		  os ?
-		      i18n("\nNext boot: %1", QString::fromLocal8Bit( os ) ) :
+		  !os.isEmpty() ?
+		      i18n("\nNext boot: %1", os ) :
 		      QString() );
 	if (timeout != TO_INF)
 		trg += i18n("\nAfter timeout: %1",
 		         force == SHUT_FORCE ?
-		             i18n("abort all sessions") :
+		             i18nc("after timeout:", "abort all sessions") :
 		             force == SHUT_FORCEMY ?
-		             i18n("abort own sessions") :
-		             i18n("cancel shutdown") );
+		             i18nc("after timeout:", "abort own sessions") :
+		             i18nc("after timeout:", "cancel shutdown") );
 	lab = new QLabel( trg, this );
 	box->addWidget( lab );
 	complete( 0 );
