@@ -25,9 +25,10 @@
 #include "kdmrect.h"
 #include "kdmlist.h"
 #include "kdmlabel.h"
+#include "kdmbutton.h"
 
-#include <kdm_greet.h>
-#include <kfdialog.h>
+#include <kdm_greet.h> // debug stuff
+#include <kfdialog.h> // kfmsgbox
 
 #include <klocale.h>
 #include <kconfig.h>
@@ -45,10 +46,9 @@
 /*
  * KdmThemer. The main theming interface
  */
-KdmThemer::KdmThemer( const QString &_filename, const QString &mode,
+KdmThemer::KdmThemer( const QString &_filename,
                       const QMap<QString, bool> &showTypes, QWidget *w )
 	: QObject()
-	, m_currentMode( mode )
 	, m_showTypes( showTypes )
 	, rootItem( 0 )
 	, m_geometryOutdated( true )
@@ -64,19 +64,19 @@ KdmThemer::KdmThemer( const QString &_filename, const QString &mode,
 	}
 	QFile opmlFile( filename );
 	if (!opmlFile.open( QIODevice::ReadOnly )) {
-		FDialog::box( w, errorbox, i18n( "Cannot open theme file %1" , filename) );
+		KFMsgBox::box( w, errorbox, i18n( "Cannot open theme file %1" , filename) );
 		return;
 	}
 	QDomDocument domTree;
 	if (!domTree.setContent( &opmlFile )) {
-		FDialog::box( w, errorbox, i18n( "Cannot parse theme file %1" , filename) );
+		KFMsgBox::box( w, errorbox, i18n( "Cannot parse theme file %1" , filename) );
 		return;
 	}
 	// generate all the items defined in the theme
 	const QDomElement &theme = domTree.documentElement();
 	// Get its tag, and check it's correct ("greeter")
 	if (theme.tagName() != "greeter") {
-		FDialog::box( w, errorbox, i18n( "%1 does not seem to be a correct theme file" , filename) );
+		KFMsgBox::box( w, errorbox, i18n( "%1 does not seem to be a correct theme file" , filename) );
 		return;
 	}
 
@@ -86,14 +86,6 @@ KdmThemer::KdmThemer( const QString &_filename, const QString &mode,
 	basedir = QFileInfo( filename ).absolutePath();
 
 	generateItems( rootItem, theme );
-	rootItem->updateVisible();
-
-/*	*TODO*
-	// Animation timer
-	QTimer *time = new QTimer( this );
-	time->start( 500 );
-	connect( time, SIGNAL(timeout()), SLOT(update()) )
-*/
 }
 
 KdmThemer::~KdmThemer()
@@ -104,7 +96,9 @@ void
 KdmThemer::setWidget( QWidget *w )
 {
 	if ((m_widget = w)) {
-		setWidgetAttribs( m_widget, rootItem->style );
+		rootItem->updateVisible();
+		m_widget->setPalette( rootItem->style.palette );
+		m_widget->setFont( rootItem->style.font.font );
 		rootItem->plugActions();
 	}
 }
@@ -169,7 +163,7 @@ KdmThemer::widgetEvent( QEvent *e )
 		break;
 	case QEvent::Paint:
 		if (m_geometryOutdated) {
-			debug() << "==== updating geometry ====" << endl;
+			debug() << "==== updating geometry ====";
 			QStack<QSize> ps;
 			QRect rect( QPoint( 0, 0 ), widget()->size() );
 			rootItem->setGeometry( ps, rect, m_geometryInvalid );
@@ -182,7 +176,7 @@ KdmThemer::widgetEvent( QEvent *e )
 			//kDebug() << "paint on: " << paintRect;
 
 			QPainter p( widget() );
-			rootItem->paint( &p, paintRect );
+			rootItem->paint( &p, paintRect, false, true );
 			rootItem->showWidget();
 		}
 		break;
@@ -192,16 +186,13 @@ KdmThemer::widgetEvent( QEvent *e )
 }
 
 void
-KdmThemer::paintBackground( QPaintDevice *dev )
+KdmThemer::paintBackground( QPaintDevice *dev, const QRect &rect, bool primaryScreen )
 {
-	if (KdmItem *bg = findNode( "background" )) {
-		debug() << "==== setting background geometry ====" << endl;
-		QRect rect( 0, 0, dev->width(), dev->height() );
-		QStack<QSize> ps;
-		bg->setGeometry( ps, rect, true );
-		QPainter p( dev );
-		bg->paint( &p, rect );
-	}
+	debug() << "==== setting background geometry ====";
+	QStack<QSize> ps;
+	rootItem->setGeometry( ps, rect, true );
+	QPainter p( dev );
+	rootItem->paint( &p, rect, true, primaryScreen );
 }
 
 void
@@ -217,37 +208,12 @@ KdmThemer::generateItems( KdmItem *parent, const QDomNode &node )
 		QString tagName = el.tagName();
 
 		if (tagName == "item") {
-			QString showType;
-			bool showTypeInvert = false;
-
-			QDomNode showNode = subnode.namedItem( "show" );
-			if (!showNode.isNull()) {
-				QDomElement sel = showNode.toElement();
-
-				QString modes = sel.attribute( "modes" );
-				if (!modes.isNull() &&
-				    (modes == "nowhere" ||
-				     (modes != "everywhere" &&
-				      !modes.split( ",", QString::SkipEmptyParts ).contains(
-				          m_currentMode ))))
-					continue;
-
-				showType = sel.attribute( "type" );
-				if (!showType.isNull()) {
-					if (showType[0] == '!') {
-						showType.remove( 0, 1 );
-						showTypeInvert = true;
-					}
-					if (!showType.startsWith( "plugin-" ) &&
-					    m_showTypes.contains( showType ) == showTypeInvert)
-						continue;
-				}
-			}
-
 			QString type = el.attribute( "type" );
 			KdmItem *newItem;
 			if (type == "label")
 				newItem = new KdmLabel( parent, subnode );
+			else if (type == "button")
+				newItem = new KdmButton( parent, subnode );
 			else if (type == "pixmap")
 				newItem = new KdmPixmap( parent, subnode );
 			else if (type == "rect")
@@ -262,8 +228,10 @@ KdmThemer::generateItems( KdmItem *parent, const QDomNode &node )
 				newItem = new KdmPixmap( parent, subnode );
 			else
 				continue;
-			newItem->setIsButton( el.attribute( "button", "false" ) == "true" );
-			newItem->setShowType( showType, showTypeInvert );
+			if (!newItem->isVisible()) {
+				delete newItem;
+				continue;
+			}
 			connect( newItem, SIGNAL(needUpdate( int, int, int, int )),
 			         SLOT(update( int, int, int, int )) );
 			connect( newItem, SIGNAL(needPlacement()),
@@ -272,8 +240,20 @@ KdmThemer::generateItems( KdmItem *parent, const QDomNode &node )
 			         SLOT(slotNeedPlugging()) );
 			connect( newItem, SIGNAL(activated( const QString & )),
 			         SIGNAL(activated( const QString & )) );
-			generateItems( newItem, subnode );
-		} else if (tagName == "box") {
+			generateLayouts( newItem, subnode );
+		}
+	}
+}
+
+void
+KdmThemer::generateLayouts( KdmItem *parent, const QDomNode &node )
+{
+	const QDomNodeList &subnodeList = node.childNodes();
+	for (int nod = 0; nod < subnodeList.count(); nod++) {
+		QDomNode subnode = subnodeList.item( nod );
+		QString tagName = subnode.toElement().tagName();
+
+		if (tagName == "box") {
 			parent->setBoxLayout( subnode );
 			generateItems( parent, subnode );
 		} else if (tagName == "fixed") {
@@ -286,7 +266,7 @@ KdmThemer::generateItems( KdmItem *parent, const QDomNode &node )
 void
 KdmThemer::showStructure()
 {
-	kDebug() << "======= item tree =======";
+	QDebug( QtDebugMsg ) << "======= item tree =======";
 	rootItem->showStructure( QString() );
 }
 
