@@ -21,6 +21,7 @@
 #include <kglobal.h>
 #include "blankscrn.h"
 #include <kaboutdata.h>
+#include <kdeversion.h>
 #include "blankscrn.moc"
 
 // libkscreensaver interface
@@ -29,8 +30,8 @@ class KBlankSaverInterface : public KScreenSaverInterface
 public:
     virtual KAboutData* aboutData()
     {
-        return new KAboutData( "kblankscrn.kss", 0, ki18n( "KBlankScreen" ),
-                               "4.0.0", ki18n( "Blank Screen Saver" ) );
+        return new KAboutData( "kblankscrn.kss", "kblankscrn", ki18n( "KBlankScreen" ),
+                               KDE_VERSION_STRING, ki18n( "Blank Screen Saver" ) );
     }
 
     virtual KScreenSaver* create( WId id )
@@ -84,6 +85,7 @@ KBlankSetup::KBlankSetup( QWidget *parent, const char *name )
 	QPalette palette;
 	palette.setColor( preview->backgroundRole(), Qt::black );
 	preview->setPalette(palette);
+	preview->setAutoFillBackground(true);
 	preview->show();    // otherwise saver does not get correct size
 	saver = new KBlankSaver( preview->winId() );
 	grid->addWidget(preview, 0, 1, 3, 1);
@@ -127,6 +129,8 @@ KBlankSaver::KBlankSaver( WId id ) : KScreenSaver( id )
 {
 	readSettings();
 	blank();
+	setAutoFillBackground(true);
+	show();
 }
 
 KBlankSaver::~KBlankSaver()
