@@ -117,6 +117,7 @@ XShmSegmentInfo SceneOpenGL::shm;
 SceneOpenGL::SceneOpenGL( Workspace* ws )
     : Scene( ws )
     , init_ok( false )
+    , selfCheckDone( false )
     {
     if( !Extensions::glxAvailable())
         {
@@ -1357,8 +1358,8 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
     {
     // check if there is something to paint (e.g. don't paint if the window
     // is only opaque and only PAINT_WINDOW_TRANSLUCENT is requested)
-    bool opaque = isOpaque() && data.opacity == 1.0;
     /* HACK: It seems this causes painting glitches, disable temporarily
+    bool opaque = isOpaque() && data.opacity == 1.0;
     if(( mask & PAINT_WINDOW_OPAQUE ) ^ ( mask & PAINT_WINDOW_TRANSLUCENT ))
         { // We are only painting either opaque OR translucent windows, not both
         if( mask & PAINT_WINDOW_OPAQUE && !opaque )
@@ -1437,20 +1438,8 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
         }
     region.translate( toplevel->x(), toplevel->y() );  // Back to screen coords
 
-    texture.bind();
-    texture.enableUnnormalizedTexCoords();
-
     WindowQuadList decoration = data.quads.select( WindowQuadDecoration );
-    // paint the content
-    if ( !(mask & PAINT_DECORATION_ONLY) )
-        {
-        prepareStates( Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
-        renderQuads( mask, region, data.quads.select( WindowQuadContents ));
-        restoreStates( Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
-        }
 
-    texture.disableUnnormalizedTexCoords();
-    texture.unbind();
 
     // decorations
     Client *client = dynamic_cast<Client*>(toplevel);
@@ -1459,10 +1448,10 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
         {
         bool noBorder = true;
         bool updateDeco = false;
-        const QPixmap *left;
-        const QPixmap *top;
-        const QPixmap *right;
-        const QPixmap *bottom;
+        const QPixmap *left = NULL;
+        const QPixmap *top = NULL;
+        const QPixmap *right = NULL;
+        const QPixmap *bottom = NULL;
         QRect topRect, leftRect, rightRect, bottomRect;
         if( client && !client->noBorder() )
             {
@@ -1490,7 +1479,7 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
             {
             WindowQuadList topList, leftList, rightList, bottomList;
 
-            foreach( WindowQuad quad, decoration )
+            foreach( const WindowQuad& quad, decoration )
                 {
                 if( topRect.contains( QPoint( quad.originalLeft(), quad.originalTop() ) ) )
                     {
@@ -1520,6 +1509,20 @@ void SceneOpenGL::Window::performPaint( int mask, QRegion region, WindowPaintDat
             }
         }
 
+    texture.bind();
+    texture.enableUnnormalizedTexCoords();
+
+    // paint the content
+    if ( !(mask & PAINT_DECORATION_ONLY) )
+        {
+        prepareStates( Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
+        renderQuads( mask, region, data.quads.select( WindowQuadContents ));
+        restoreStates( Content, data.opacity * data.contents_opacity, data.brightness, data.saturation, data.shader );
+        }
+
+    texture.disableUnnormalizedTexCoords();
+    texture.unbind();
+
     glPopMatrix();
     }
 
@@ -1550,7 +1553,7 @@ void SceneOpenGL::Window::paintDecoration( const QPixmap* decoration, TextureTyp
         // texture doesn't need updating, just bind it
         glBindTexture( decorationTexture->target(), decorationTexture->texture());
         }
-    else
+    else if( !decoration->isNull() )
         {
         bool success = decorationTexture->load( decoration->handle(), decoration->size(), decoration->depth() );
         if( !success )
@@ -1559,6 +1562,8 @@ void SceneOpenGL::Window::paintDecoration( const QPixmap* decoration, TextureTyp
             return;
             }
         }
+    else
+        return;
     if( filter == ImageFilterGood )
         {
         // avoid unneeded mipmap generation by only using trilinear
@@ -1610,7 +1615,7 @@ void SceneOpenGL::Window::makeDecorationArrays( float** vertices, float** texcoo
     *texcoords = new float[ quads.count() * 4 * 2 ];
     float* vpos = *vertices;
     float* tpos = *texcoords;
-    foreach( WindowQuad quad, quads )
+    foreach( const WindowQuad& quad, quads )
         {
         *vpos++ = quad[ 0 ].x();
         *vpos++ = quad[ 0 ].y();
